import os
import pathlib
import sys

run_locally = os.getenv('PYPWS_RUN_LOCALLY')
if run_locally and run_locally.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')

from pypws.calculations import UserDefinedSourceLinkedRunCalculation
from pypws.entities import Material, MaterialComponent, Weather, Substrate, DischargeParameters, FlammableParameters, DispersionParameters, FlammableParameters, ExplosionParameters, ExplosionOutputConfig, ExplosionConfinedVolume, FlammableOutputConfig, DispersionOutputConfig, ExplosionConfinedVolume, State, DischargeRecord, DischargeResult, DischargeParameters, DispersionParameters, DispersionOutputConfig, ExplosionOutputConfig, FlammableOutputConfig
from pypws.enums import ResultCode, AtmosphericStabilityClass, SpecialConcentration, WindProfileFlag, Phase, FluidSpec, DynamicType

def test_vlr_153():

    material = Material("AMMONIA", [MaterialComponent("AMMONIA", 1.0)], component_count = 1)

    # Define the weather conditions
    weather = Weather(wind_speed=5.0, stability_class=AtmosphericStabilityClass.STABILITY_B, wind_profile_flag=WindProfileFlag.LOGARITHMIC_PROFILE)

    # Define the substrate
    substrate = Substrate(surface_roughness=0.01)

    # Define the dispersion parameters
    dispersion_parameters = [DispersionParameters(averaging_time=18.75), DispersionParameters(averaging_time=600.0)]

    # Define the dispersion output configuration
    dispersion_output_configs_flammable = [DispersionOutputConfig(special_concentration=SpecialConcentration.LFL_FRACTION, elevation=0.0)]
    dispersion_output_configs_toxic = [DispersionOutputConfig(special_concentration=SpecialConcentration.NOT_DEFINED, concentration=5e-5, elevation=0.0)]

    # Define the flammable parameters
    flammable_parameters = FlammableParameters()

    # Define the flammable output configuration
    flammable_output_configs = [FlammableOutputConfig()]

    # Define the explosion parameters
    explosion_parameters = ExplosionParameters()

    # Define the explosion output configuration
    explosion_output_configs = [ExplosionOutputConfig()]

    # Define the explosion confined volumes
    explosion_confined_volumes = [ExplosionConfinedVolume()]

    user_defined_source_linked_run_calculation = UserDefinedSourceLinkedRunCalculation(
        material=material,
        phase_to_be_released= Phase.LIQUID,
        discharge_records = [DischargeRecord(mass_flow = 50.0, time = 600.0, final_state = State(temperature =200.0, pressure = 101325.0, flash_flag = FluidSpec.TP, liquid_fraction= 1.0), orifice_state=State(pressure = 101325, temperature=300, liquid_fraction= 1.0), orifice_velocity= 0.0, storage_state=State(pressure = 101325, temperature=300, liquid_fraction=1.0), final_velocity= 60.0, droplet_diameter = 0.0009)],
        discharge_record_count = 1, 
        discharge_result= DischargeResult(release_mass = 3000.0, height = 0.0, angle = 0.0, hole_diameter= 0.3, release_type=DynamicType.CONTINUOUS, pre_dilution_air_rate=0.0, expansion_energy=6000.0),
        discharge_parameters= DischargeParameters(),
        substrate= substrate,
        weather= weather,
        dispersion_parameters= dispersion_parameters,
        dispersion_parameter_count=2,
        flammable_parameters= flammable_parameters,
        explosion_parameters= explosion_parameters,
        explosion_output_configs= explosion_output_configs,
        explosion_output_config_count= 1,
        explosion_confined_volumes= explosion_confined_volumes,
        explosion_confined_volume_count = 1,
        flammable_output_configs= flammable_output_configs,
        flammable_output_config_count = 1,
        dispersion_flam_output_configs=dispersion_output_configs_flammable,
        dispersion_flam_output_config_count= 1,
        dispersion_toxic_output_configs= dispersion_output_configs_toxic,
        dispersion_toxic_output_config_count= 1,
        end_point_concentration=0.0
        )                                   
        
    # Run the calculation
    print('Running user_defined_source_linked_run_calculation...')
    resultCode = user_defined_source_linked_run_calculation.run()

    # Print any messages.
    if len(user_defined_source_linked_run_calculation.messages) > 0:
        print('Messages:')
        for message in user_defined_source_linked_run_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        print(f'SUCCESS: user_defined_source_linked_run_calculation ({user_defined_source_linked_run_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED user_defined_source_linked_run_calculation with result code {resultCode}'
    if resultCode == ResultCode.SUCCESS:
        if (user_defined_source_linked_run_calculation.jet_fire_flame_result.flame_length != 0.0):
            assert False,f'Regression failed with user_defined_source_linked_run_calculation.jet_fire_flame_result.flame_length = {user_defined_source_linked_run_calculation.jet_fire_flame_result.flame_length}'
        if (abs(user_defined_source_linked_run_calculation.pool_fire_flame_result.flame_diameter -87.63609313964844)/87.63609313964844 > 1e-3):
            assert False,f'Regression failed with user_defined_source_linked_run_calculation.pool_fire_flame_result.flame_diameter = {user_defined_source_linked_run_calculation.pool_fire_flame_result.flame_diameter}'
        if (len(user_defined_source_linked_run_calculation.flam_conc_contour_points) != 0):
            assert False,f'Regression failed with len(user_defined_source_linked_run_calculation.flam_conc_contour_points) = {len(user_defined_source_linked_run_calculation.flam_conc_contour_points)}'
        if (len(user_defined_source_linked_run_calculation.toxic_conc_contour_points) != 637):
            assert False,f'Regression failed with len(user_defined_source_linked_run_calculation.toxic_conc_contour_points) = {len(user_defined_source_linked_run_calculation.toxic_conc_contour_points)}'
        if (user_defined_source_linked_run_calculation.area_footprint_flam_conc[0] != 0.0):
            assert False,f'Regression failed with user_defined_source_linked_run_calculation.area_footprint_flam_conc[0] = {user_defined_source_linked_run_calculation.area_footprint_flam_conc[0]}'
        if (abs(user_defined_source_linked_run_calculation.area_footprint_toxic_conc[0] - 940696.3465428002)/940696.3465428002 > 1e-3):
            assert False,f'Regression failed with user_defined_source_linked_run_calculation.area_footprint_toxic_conc[0] = {user_defined_source_linked_run_calculation.area_footprint_toxic_conc[0]}'
        if (user_defined_source_linked_run_calculation.area_contour_jet[0] != 0.0):
            assert False,f'Regression failed with user_defined_source_linked_run_calculation.area_contour_jet[0] = {user_defined_source_linked_run_calculation.area_contour_jet[0]}'
        if (abs(user_defined_source_linked_run_calculation.area_contour_pool[0] -25251.117265661516 )/25251.117265661516 >1e-3):
            assert False,f'Regression failed with user_defined_source_linked_run_calculation.area_contour_pool[0] = {user_defined_source_linked_run_calculation.area_contour_pool[0]}'
        if (user_defined_source_linked_run_calculation.explosion_overpressure_results[0].exploded_mass != 0.0):
            assert False,f'Regression failed with user_defined_source_linked_run_calculation.explosion_overpressure_results[0].exploded_mass = {user_defined_source_linked_run_calculation.explosion_overpressure_results[0].exploded_mass}'
        if (user_defined_source_linked_run_calculation.explosion_overpressure_results[0].maximum_distance != 0.0):
            assert False,f'Regression failed with user_defined_source_linked_run_calculation.explosion_overpressure_results[0].maximum_distance = {user_defined_source_linked_run_calculation.explosion_overpressure_results[0].maximum_distance}'
        print(f'Flame lenght = {user_defined_source_linked_run_calculation.jet_fire_flame_result.flame_length}')
        print(f'Flame diameter = {user_defined_source_linked_run_calculation.pool_fire_flame_result.flame_diameter}')
        print(f'Length of the dispersion flammable contour array = {len(user_defined_source_linked_run_calculation.flam_conc_contour_points)}')
        print(f'Length of the dispersion toxic contour array = {len(user_defined_source_linked_run_calculation.toxic_conc_contour_points)}')
        print(f'Area of the flammable cloud = {user_defined_source_linked_run_calculation.area_footprint_flam_conc[0]}')
        print(f'Area of the toxic cloud = {user_defined_source_linked_run_calculation.area_footprint_toxic_conc[0]}')
        print(f'Area of the jet fire = {user_defined_source_linked_run_calculation.area_contour_jet[0]}')
        print(f'Area of the pool = {user_defined_source_linked_run_calculation.area_contour_pool[0]}')
        print(f'Flammable mass = {user_defined_source_linked_run_calculation.explosion_overpressure_results[0].exploded_mass}')
        print(f'Explosion maximum distance = {user_defined_source_linked_run_calculation.explosion_overpressure_results[0].maximum_distance}')
        print(f'SUCCESS: user_defined_source_linked_run_calculation ({user_defined_source_linked_run_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED user_defined_source_linked_run_calculation with result code {resultCode}'


import os
import pathlib
import sys

# When running locally the environment variable PYPWS_RUN_LOCALLY needs to be set to True.
# Check if the environment variable is set
PYPWS_RUN_LOCALLY = os.getenv('PYPWS_RUN_LOCALLY')
if PYPWS_RUN_LOCALLY and PYPWS_RUN_LOCALLY.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')


from pypws.calculations import MixtureConstantPropertiesCalculation
from pypws.entities import Material, MaterialComponent, State
from pypws.enums import ResultCode


def test_case_97():

    # Set the material
    material = Material("NATURAL GAS", [MaterialComponent("METHANE", 0.85), MaterialComponent("ETHANE", 0.1), MaterialComponent("PROPANE", 0.05)], component_count = 3)

    # Create a mixture constant properties calculation using the material.
    mixture_constant_properties_calculation = MixtureConstantPropertiesCalculation(material)

    # Run the calculation
    print('Running mixture_constant_properties_calculation...')
    resultCode = mixture_constant_properties_calculation.run()

    # Print any messages.
    if len(mixture_constant_properties_calculation.messages) > 0:
        print('Messages:')
        for message in mixture_constant_properties_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        if (abs((mixture_constant_properties_calculation.mix_constant_prop_result.lower_flammability_limit - 0.04402515723270441) / 0.04402515723270441) > 1e-3):
            assert False, f'Regression failed with mixture_constant_properties_calculation lower_flammability_limit = {mixture_constant_properties_calculation.mix_constant_prop_result.lower_flammability_limit}'
        if (abs((mixture_constant_properties_calculation.mix_constant_prop_result.upper_flammability_limit - 0.14286869340232858) / 0.14286869340232858) > 1e-3):
            assert False, f'Regression failed with mixture_constant_properties_calculation upper_flammability_limit = {mixture_constant_properties_calculation.mix_constant_prop_result.upper_flammability_limit}'
        if (abs((mixture_constant_properties_calculation.mix_constant_prop_result.critical_pressure - 4608750.0) / 4608750.0) > 1e-3):
            assert False, f'Regression failed with mixture_constant_properties_calculation critical_pressure = {mixture_constant_properties_calculation.mix_constant_prop_result.critical_pressure}'
        if (abs((mixture_constant_properties_calculation.mix_constant_prop_result.critical_temperature - 210.991) / 210.991) > 1e-3):
            assert False, f'Regression failed with mixture_constant_properties_calculation critical_temperature = {mixture_constant_properties_calculation.mix_constant_prop_result.critical_temperature}'
        if (abs((mixture_constant_properties_calculation.mix_constant_prop_result.molecular_weight - 18.847806000000002) / 18.847806000000002) > 1e-3):
            assert False, f'Regression failed with mixture_constant_properties_calculation molecular_weight = {mixture_constant_properties_calculation.mix_constant_prop_result.molecular_weight}'
        if (abs((mixture_constant_properties_calculation.mix_constant_prop_result.bubble_point - 113.67514401917592) / 113.67514401917592) > 1e-3):
            assert False, f'Regression failed with mixture_constant_properties_calculation bubble_point = {mixture_constant_properties_calculation.mix_constant_prop_result.bubble_point}'
        print(f'SUCCESS: mixture_constant_properties_calculation ({mixture_constant_properties_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED mixture_constant_properties_calculation with result code {resultCode}'


import os
import pathlib
import sys

run_locally = os.getenv('PYPWS_RUN_LOCALLY')
if run_locally and run_locally.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')

from pypws.calculations import LoadMassInventoryVesselForReliefValveScenarioCalculation
from pypws.entities import Material, MaterialComponent
from pypws.enums import ResultCode


def test_case_104():

	material = Material("METHANE+ETHANE", [MaterialComponent("METHANE", 0.9), MaterialComponent("ETHANE", 0.1)], component_count = 2)


	# Create a load mass inventory vessel for relief valve scenario calculation using the material.
	load_mass_inventory_vessel_for_relief_valve_scenario_calculation = LoadMassInventoryVesselForReliefValveScenarioCalculation(material = material, temperature = 250, pressure = float(11e5), mass = float(1e5), pipe_length = 10.0, pipe_diameter = 0.5, constriction_size = 0.5, release_elevation = 100.0, release_angle = 0.0)

	# Run the calculation
	print('Running load_mass_inventory_vessel_for_relief_valve_scenario_calculation...')
	resultCode = load_mass_inventory_vessel_for_relief_valve_scenario_calculation.run()

	# Print any messages.
	if len(load_mass_inventory_vessel_for_relief_valve_scenario_calculation.messages) > 0:
		print('Messages:')
		for message in load_mass_inventory_vessel_for_relief_valve_scenario_calculation.messages:
			print(message)

	if resultCode == ResultCode.SUCCESS:
		if (abs((load_mass_inventory_vessel_for_relief_valve_scenario_calculation.vessel.diameter-27.049130317992706)/27.049130317992706)>1e-3):
			assert False, f'Regression failed with load_mass_inventory_vessel_for_relief_valve_scenario_calculation.vessel.diameter = {load_mass_inventory_vessel_for_relief_valve_scenario_calculation.vessel.diameter}'
		if (abs(((load_mass_inventory_vessel_for_relief_valve_scenario_calculation.vessel.location.z-86.47543484100365))/(86.47543484100365))>1e-3):
			assert False, f'Regression failed load_mass_inventory_vessel_for_relief_valve_scenario_calculation.vessel.location.z = {load_mass_inventory_vessel_for_relief_valve_scenario_calculation.vessel.location.z}'
		if (abs((load_mass_inventory_vessel_for_relief_valve_scenario_calculation.relief_valve.pipe_diameter-0.5)/0.5)>1e-3):
			assert False, f'Regression failed with load_mass_inventory_vessel_for_relief_valve_scenario_calculation.relief_valve.pipe_diameter = {load_mass_inventory_vessel_for_relief_valve_scenario_calculation.relief_valve.pipe_diameter}'
		if (abs((load_mass_inventory_vessel_for_relief_valve_scenario_calculation.relief_valve.pipe_height_fraction-0.5)/0.5)>1e-3):
			assert False, f'Regression failed with load_mass_inventory_vessel_for_relief_valve_scenario_calculation.relief_valve.pipe_height_fraction = {load_mass_inventory_vessel_for_relief_valve_scenario_calculation.relief_valve.pipe_height_fraction}'
		print(f'SUCCESS: load_mass_inventory_vessel_for_relief_valve_scenario_calculation ({load_mass_inventory_vessel_for_relief_valve_scenario_calculation.calculation_elapsed_time}ms)')

	else:
		assert False, f'FAILED load_mass_inventory_vessel_for_relief_valve_scenario_calculation with result code {resultCode}'





# ********************************************************************************
#
# This file has been auto-generated by a code generation tool.
# Version:   4.2.11
# Date/time: 01 May 2025 14:41:44
# Template:  templates/pythonpws/version.razor.
#
#
#  Copyright (c) DNV AS. All rights reserved.
#
#  This source code is subject to copyright protection under international
#  copyright laws.  Reproduction, disclosure, distribution or use of this source
#  code is prohibited without the express written permission of DNV AS.
#
# ********************************************************************************

__version__ = "4.2.11"


import os
import pathlib
import sys

PYPWS_RUN_LOCALLY = os.getenv('PYPWS_RUN_LOCALLY')
if PYPWS_RUN_LOCALLY and PYPWS_RUN_LOCALLY.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')


from pypws.calculations import (
    DispersionCalculation,
    PoolFireCalculation,
    RadiationAtPointsForPoolFiresCalculation,
    VesselCatastrophicRuptureCalculation,
    VesselStateCalculation,
)
from pypws.entities import (
    Bund,
    DischargeParameters,
    DispersionParameters,
    FlammableOutputConfig,
    FlammableParameters,
    LocalPosition,
    Material,
    MaterialComponent,
    State,
    Substrate,
    Vessel,
    Weather,
)
from pypws.enums import (
    AtmosphericStabilityClass,
    PoolFireType,
    ResultCode,
    WindProfileFlag
)


def test_41a():

    # Define the material contained by the vessel.
    material = Material("N-HEXANE", [MaterialComponent("N-HEXANE", 1.0)])

    # Define the initial state of the vessel.
    state = State(temperature=280.0, pressure= 2.0e5, liquid_fraction=0.8)

    # Create a vessel state calculation using the material and state.
    vessel_state_calculation = VesselStateCalculation(material, state)

    # Run the vessel state calculation.
    print('Running vessel_state_calculation...')
    resultCode = vessel_state_calculation.run()

    # Print any messages.
    if len(vessel_state_calculation.messages) > 0:
        print('Messages:')
        for message in vessel_state_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        print(f'SUCCESS: vessel_state_calculation ({vessel_state_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED vessel_state_calculation with result code {resultCode}'

    # Create a vessel to use in the vessel catastrophic rupture calculation using the previously defined entities.
    vessel = Vessel(state=state,
                    material=material,
                    liquid_fill_fraction_by_volume=0.8,
                    vessel_conditions = vessel_state_calculation.vessel_conditions
                    )

    # Define the discharge parameters for the vessel catastrophic rupture calculation.
    discharge_parameters = DischargeParameters()

    # Create a vessel catastrophic rupture calculation using the vessel and discharge parameters.
    vessel_catastrophic_rupture_calculation = VesselCatastrophicRuptureCalculation(vessel, discharge_parameters)

    # Run the calculation.
    print('Running vessel_catastrophic_rupture_calculation...')
    resultCode = vessel_catastrophic_rupture_calculation.run()

    # Print any messages.
    if len(vessel_catastrophic_rupture_calculation.messages) > 0:
        print('Messages:')
        for message in vessel_catastrophic_rupture_calculation.messages:
            print(message)

    if resultCode == resultCode.SUCCESS:
        print(f'SUCCESS: vessel_catastrophic_rupture_calculation ({vessel_catastrophic_rupture_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED vessel_catastrophic_rupture_calculation with result code {resultCode}'

    # Define the weather.
    weather = Weather(wind_speed = 1.5, stability_class = AtmosphericStabilityClass.STABILITY_F, wind_profile_flag = WindProfileFlag.LOGARITHMIC_PROFILE)

    # Define the substrate.
    substrate = Substrate(bund = Bund())

    # Create a dispersion calculation based on the vessel catastrophic rupture calculation, weather, substrate, and dispersion parameters.
    dispersion_calculation = DispersionCalculation(discharge_records = vessel_catastrophic_rupture_calculation.discharge_records, discharge_result = vessel_catastrophic_rupture_calculation.discharge_result, weather = weather, substrate = substrate, dispersion_parameters = DispersionParameters(), end_point_concentration = 0.0, discharge_record_count = 1, material = vessel_catastrophic_rupture_calculation.exit_material)

    # Run the calculation.
    print('Running dispersion_calculation...')
    resultCode = dispersion_calculation.run()

    # Print any messages.
    if len(dispersion_calculation.messages) > 0:
        print('Messages:')
        for message in dispersion_calculation.messages:
            print(message)

    if resultCode == resultCode.SUCCESS:
        print(f'SUCCESS: dispersion_calculation_calculation ({dispersion_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED dispersion_calculation with result code {resultCode}'

    # Define a flammable parameter set.
    flammable_parameters = FlammableParameters(pool_fire_type = PoolFireType.LATE)

    # Create a pool fire calculation based on the dispersion calculation, weather, substrate, and flammable parameters.
    pool_fire_calculation = PoolFireCalculation(material = vessel_catastrophic_rupture_calculation.exit_material, pool_records = dispersion_calculation.pool_records, pool_record_count = len(dispersion_calculation.pool_records), weather = weather, substrate = substrate, flammable_parameters = flammable_parameters)

    # Run the calculation.
    print('Running pool_fire_calculation...')
    resultCode = pool_fire_calculation.run()

    # Print any messages.
    if len(pool_fire_calculation.messages) > 0:
        print('Messages:')
        for message in pool_fire_calculation.messages:
            print(message)

    if resultCode == resultCode.SUCCESS:
        print(f'SUCCESS: pool_fire_calculation ({pool_fire_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED pool_fire_calculation with result code {resultCode}'

    # Define a flammable output config.
    flammable_output_configs = [FlammableOutputConfig(position = LocalPosition(50.0, 0.0, 0.0)), FlammableOutputConfig(position = LocalPosition(50.0, 0.0, 1.0)), FlammableOutputConfig(position = LocalPosition(50.0, 3.0, 2.0))]

    # Create a radiation a points calculation based on the pool fire calculation, weather, substrate, and flammable output config.
    radiation_at_points_for_pool_fires_calculation = RadiationAtPointsForPoolFiresCalculation(flame_records = pool_fire_calculation.flame_records, pool_fire_flame_result = pool_fire_calculation.pool_fire_flame_result, flame_record_count = len(pool_fire_calculation.flame_records), flammable_output_configs = flammable_output_configs, weather = weather, flammable_parameters = flammable_parameters, flammable_output_config_count = len (flammable_output_configs))    

    # Run the calculation.
    print('Running radiation_at_points_for_pool_fires_calculation...')
    resultCode = radiation_at_points_for_pool_fires_calculation.run()

    # Print any messages.
    if len(radiation_at_points_for_pool_fires_calculation.messages) > 0:
        print('Messages:')
        for message in radiation_at_points_for_pool_fires_calculation.messages:
            print(message)

    if resultCode == resultCode.SUCCESS:
        if (abs((radiation_at_points_for_pool_fires_calculation.radiation[0]-11159.009681754622)/11159.009681754622)>1e-3):
            assert False,f'Regression failed with radiation value {radiation_at_points_for_pool_fires_calculation.radiation[0]}'
        if (abs((radiation_at_points_for_pool_fires_calculation.pool_fire_flame_result.flame_diameter-50.3738899230957)/50.3738899230957)>1e-3):
            assert False,f'Regression failed with flame diameter value {radiation_at_points_for_pool_fires_calculation.pool_fire_flame_result.flame_diameter}'
        print(f'SUCCESS: radiation_at_points_for_pool_fires_calculation ({radiation_at_points_for_pool_fires_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED radiation_at_points_for_pool_fires_calculation with result code {resultCode}'



import os
import pathlib
import sys

# When running locally the environment variable PYPWS_RUN_LOCALLY needs to be set to True.
# Check if the environment variable is set
PYPWS_RUN_LOCALLY = os.getenv('PYPWS_RUN_LOCALLY')
if PYPWS_RUN_LOCALLY and PYPWS_RUN_LOCALLY.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')

from pypws.utilities import single

def test_single_single_match():
    assert single([1, 2, 3, 4, 5], lambda x: x == 3) == 3



import os
import pathlib
import sys

run_locally = os.getenv('PYPWS_RUN_LOCALLY')
if run_locally and run_locally.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')

from pypws.calculations import LoadMassInventoryVesselForLeakScenarioCalculation, VesselLeakLinkedRunCalculation
from pypws.entities import Material, MaterialComponent, Weather, Substrate, DischargeParameters, LineRupture, Vessel, FlammableParameters, DispersionParameters, FlammableParameters, ExplosionParameters, ExplosionOutputConfig, ExplosionConfinedVolume, FlammableOutputConfig, DispersionOutputConfig, ExplosionConfinedVolume
from pypws.enums import ResultCode, AtmosphericStabilityClass, SpecialConcentration, WindProfileFlag


def test_vlr_157():

    material = Material("AMMONIA", [MaterialComponent("AMMONIA", 1.0)], component_count = 1)


    # Create a load mass inventory vessel for leak scenario calculation using the material.
    load_mass_inventory_vessel_for_leak_scenario_calculation = LoadMassInventoryVesselForLeakScenarioCalculation(material = material, temperature = 250, pressure = float(11e5), mass = float(1e3), hole_size= 0.008, release_elevation = 1.0, release_angle = 0.35)

    # Run the calculation
    print('Running load_mass_inventory_vessel_for_leak_scenario_calculation...')
    resultCode = load_mass_inventory_vessel_for_leak_scenario_calculation.run()

    # Print any messages.
    if len(load_mass_inventory_vessel_for_leak_scenario_calculation.messages) > 0:
        print('Messages:')
        for message in load_mass_inventory_vessel_for_leak_scenario_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        print(f'SUCCESS: load_mass_inventory_vessel_for_leak_scenario_calculation ({load_mass_inventory_vessel_for_leak_scenario_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED load_mass_inventory_vessel_for_leak_scenario_calculation with result code {resultCode}'

    # Define the weather conditions
    weather = Weather( wind_speed = 5.0, stability_class = AtmosphericStabilityClass.STABILITY_B, wind_profile_flag = WindProfileFlag.LOGARITHMIC_PROFILE)

    # Define the substrate
    substrate = Substrate()

    # Define the dispersion parameters
    dispersion_parameters = [DispersionParameters(averaging_time = 18.75), DispersionParameters(averaging_time = 18.75)]

    # Define the dispersion output configuration
    dispersion_output_configs_flammable = [DispersionOutputConfig(special_concentration = SpecialConcentration.LFL_FRACTION, elevation = 0.0)]
    dispersion_output_configs_toxic = [DispersionOutputConfig(special_concentration = SpecialConcentration.NOT_DEFINED, concentration = 5e-5, elevation = 0.0)]

    # Define the flammable parameters
    flammable_parameters = FlammableParameters()

    # Define the flammable output configuration
    flammable_output_configs = [FlammableOutputConfig()]

    # Define the explosion parameters
    explosion_parameters = ExplosionParameters()

    # Define the explosion output configuration
    explosion_output_configs = [ExplosionOutputConfig()]

    # Define the explosion confined volumes
    explosion_confined_volumes = [ExplosionConfinedVolume()]

    #
    vessel_leak_linked_run_calculation = VesselLeakLinkedRunCalculation(
        vessel = load_mass_inventory_vessel_for_leak_scenario_calculation.vessel,
        leak = load_mass_inventory_vessel_for_leak_scenario_calculation.leak,
        discharge_parameters = DischargeParameters(),
        substrate = substrate,
        weather = weather,
        dispersion_parameters = dispersion_parameters,
        dispersion_parameter_count = len(dispersion_parameters),
        end_point_concentration = 0.0,
        flammable_parameters = flammable_parameters,
        explosion_parameters = explosion_parameters,
        dispersion_flam_output_configs = dispersion_output_configs_flammable,
        dispersion_flam_output_config_count = len(dispersion_output_configs_flammable),
        dispersion_toxic_output_configs = dispersion_output_configs_toxic,
        dispersion_toxic_output_config_count = len(dispersion_output_configs_toxic),
        flammable_output_configs = flammable_output_configs,
        flammable_output_config_count = len(flammable_output_configs),
        explosion_output_configs = explosion_output_configs,
        explosion_output_config_count = len(explosion_output_configs),
        explosion_confined_volumes = explosion_confined_volumes,
        explosion_confined_volume_count = len(explosion_confined_volumes)
    )

    # Run the calculation
    print('Running vessel_leak_linked_run_calculation...')
    resultCode = vessel_leak_linked_run_calculation.run()

    # Print any messages.
    if len(vessel_leak_linked_run_calculation.messages) > 0:
        print('Messages:')
        for message in vessel_leak_linked_run_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        if (abs(vessel_leak_linked_run_calculation.discharge_record.mass_flow - 1.2425987768572015)/1.2425987768572015 > 1e-3):
            assert False,f'Regression failed with vessel_leak_linked_run_calculation.discharge_record.mass_flow = {vessel_leak_linked_run_calculation.discharge_record.mass_flow}'
        if (abs(vessel_leak_linked_run_calculation.jet_fire_flame_result.flame_length - 20.59055621597361)/20.59055621597361 > 1e-3):
            assert False,f'Regression failed with vessel_leak_linked_run_calculation.jet_fire_flame_result.flame_length = {vessel_leak_linked_run_calculation.jet_fire_flame_result.flame_length}'
        if (vessel_leak_linked_run_calculation.pool_fire_flame_result.flame_diameter != 0.0):
            assert False,f'Regression failed with vessel_leak_linked_run_calculation.pool_fire_flame_result.flame_diameter = {vessel_leak_linked_run_calculation.pool_fire_flame_result.flame_diameter}'
        if (len(vessel_leak_linked_run_calculation.flam_conc_contour_points) != 0):
            assert False,f'Regression failed with len(vessel_leak_linked_run_calculation.flam_conc_contour_points) = {len(vessel_leak_linked_run_calculation.flam_conc_contour_points)}'
        if (len(vessel_leak_linked_run_calculation.toxic_conc_contour_points) != 603):
            assert False,f'Regression failed with len(vessel_leak_linked_run_calculation.toxic_conc_contour_points) = {len(vessel_leak_linked_run_calculation.toxic_conc_contour_points)}'
        if (vessel_leak_linked_run_calculation.area_footprint_flam_conc[0] != 0.0):
            assert False,f'Regression failed with vessel_leak_linked_run_calculation.area_footprint_flam_conc[0] = {vessel_leak_linked_run_calculation.area_footprint_flam_conc[0]}'
        if (abs(vessel_leak_linked_run_calculation.area_footprint_toxic_conc[0] - 28937.347548819664)/28937.347548819664 > 1e-3):
            assert False,f'Regression failed with vessel_leak_linked_run_calculation.area_footprint_toxic_conc[0] = {vessel_leak_linked_run_calculation.area_footprint_toxic_conc[0]}'
        if (abs(vessel_leak_linked_run_calculation.area_contour_jet[0] - 239.35604734762828)/239.35604734762828 > 1e-3):
            assert False,f'Regression failed with vessel_leak_linked_run_calculation.area_contour_jet[0] = {vessel_leak_linked_run_calculation.area_contour_jet[0]}'
        if (vessel_leak_linked_run_calculation.area_contour_pool[0] != 0.0):
            assert False,f'Regression failed with vessel_leak_linked_run_calculation.area_contour_pool[0] = {vessel_leak_linked_run_calculation.area_contour_pool[0]}'
        if (vessel_leak_linked_run_calculation.explosion_overpressure_results[0].exploded_mass != 0.0):
            assert False,f'Regression failed with vessel_leak_linked_run_calculation.explosion_overpressure_results[0].exploded_mass = {vessel_leak_linked_run_calculation.explosion_overpressure_results[0].exploded_mass}'
        if (vessel_leak_linked_run_calculation.explosion_overpressure_results[0].maximum_distance != 0.0):
            assert False,f'Regression failed with vessel_leak_linked_run_calculation.explosion_overpressure_results[0].maximum_distance = {vessel_leak_linked_run_calculation.explosion_overpressure_results[0].maximum_distance}'
        print(f'Mass flow ={vessel_leak_linked_run_calculation.discharge_record.mass_flow}')
        print(f'Flame lenght = {vessel_leak_linked_run_calculation.jet_fire_flame_result.flame_length}')
        print(f'Flame diameter = {vessel_leak_linked_run_calculation.pool_fire_flame_result.flame_diameter}')
        print(f'Length of the dispersion flammable contour array = {len(vessel_leak_linked_run_calculation.flam_conc_contour_points)}')
        print(f'Length of the dispersion toxic contour array = {len(vessel_leak_linked_run_calculation.toxic_conc_contour_points)}')
        print(f'Area of the flammable cloud = {vessel_leak_linked_run_calculation.area_footprint_flam_conc[0]}')
        print(f'Area of the toxic cloud = {vessel_leak_linked_run_calculation.area_footprint_toxic_conc[0]}')
        print(f'Area of the jet fire = {vessel_leak_linked_run_calculation.area_contour_jet[0]}')
        print(f'Area of the pool = {vessel_leak_linked_run_calculation.area_contour_pool[0]}')
        print(f'Flammable mass = {vessel_leak_linked_run_calculation.explosion_overpressure_results[0].exploded_mass}')
        print(f'Explosion maximum distance = {vessel_leak_linked_run_calculation.explosion_overpressure_results[0].maximum_distance}')
        print(f'SUCCESS: vessel_leak_linked_run_calculation ({vessel_leak_linked_run_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED vessel_leak_linked_run_calculation with result code {resultCode}'


import os
import pathlib
import sys

# When running locally the environment variable PYPWS_RUN_LOCALLY needs to be set to True.
# Check if the environment variable is set
PYPWS_RUN_LOCALLY = os.getenv('PYPWS_RUN_LOCALLY')
if PYPWS_RUN_LOCALLY and PYPWS_RUN_LOCALLY.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')

from pypws.calculations import VesselLineRuptureCalculation, VesselStateCalculation
from pypws.entities import (
    DischargeParameters,
    LineRupture,
    LocalPosition,
    Material,
    MaterialComponent,
    State,
    TimeVaryingOption,
    Vessel,
    VesselConditions,
    VesselShape,
)
from pypws.enums import ResultCode

"""
This sample demonstrates how to use the vessel line rupture calculation along with with the dependent entities.
"""

def test_vlr_propane():

    # Define the initial state of the vessel.
    state = State(temperature=250.0, pressure=float(3.0e5), liquid_fraction=1.0)

    # Define the material contained by the vessel.
    material = Material("PROPANE", [MaterialComponent("PROPANE", 1.0)])

    # Create a vessel state calculation using the material and state.
    vessel_state_calculation = VesselStateCalculation(material, state)

    # Run the vessel state calculation.
    print('Running vessel_state_calculation...')
    resultCode = vessel_state_calculation.run()

    # Print any messages.
    if len(vessel_state_calculation.messages) > 0:
        print('Messages:')
        for message in vessel_state_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        print(f'SUCCESS: vessel_state_calculation ({vessel_state_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED vessel_state_calculation with result code {resultCode}'

    # Create a vessel to use in the line rupture calculation using the previously defined entities. The vessel is a horizontal cylinder with a diameter of 2m and a length of 5m.
    vessel = Vessel(state=state, material=material, vessel_conditions=vessel_state_calculation.vessel_conditions, liquid_fill_fraction_by_volume=0.7, shape=VesselShape.HORIZONTAL_CYLINDER, diameter=2, length=5)

    # Create a line rupture to use in the vessel line rupture calculation.Pipe diameter is 0.1m, pipe length is 5m, and pipe height fraction is 0.3.
    line_rupture = LineRupture(pipe_diameter=0.1, pipe_length=5.0, pipe_height_fraction=0.3)

    # Create a vessel line rupture calculation using the vessel, line rupture, and discharge parameters.
    vessel_line_rupture_calculation = VesselLineRuptureCalculation(vessel, line_rupture, DischargeParameters())

    # Run a vessel line rupture calculation.
    print('Running vessel_line_rupture_calculation...')
    resultCode = vessel_line_rupture_calculation.run()

    assert resultCode == ResultCode.SUCCESS

    # Print any messages.
    if len(vessel_line_rupture_calculation.messages) > 0:
        print('Messages:')
        for message in vessel_line_rupture_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        if (len(vessel_line_rupture_calculation.discharge_records) != 2):
            assert False,f'Regression failed with len(vessel_line_rupture_calculation.discharge_records) = {len(vessel_line_rupture_calculation.discharge_records)}'
        if (abs((vessel_line_rupture_calculation.discharge_records[0].mass_flow - 67.02362285598161) / 67.02362285598161) > 1e-3):
            assert False,f'Regression failed with vessel_line_rupture_calculation.discharge_records[0].mass_flow = {vessel_line_rupture_calculation.discharge_records[0].mass_flow}'
        if (abs((vessel_line_rupture_calculation.discharge_records[0].final_state.temperature -231.07970451768477) / 231.07970451768477) > 1e-3):
            assert False,f'Regression failed with vessel_line_rupture_calculation.discharge_records[0].final_state.temperature = {vessel_line_rupture_calculation.discharge_records[0].final_state.temperature}'
        if (abs((vessel_line_rupture_calculation.discharge_records[0].final_velocity - 63.51263298624852) / 63.51263298624852) > 1e-3):
            assert False,f'Regression failed with vessel_line_rupture_calculation.discharge_records[0].final_velocity = {vessel_line_rupture_calculation.discharge_records[0].final_velocity}'
        if (abs((vessel_line_rupture_calculation.discharge_result.release_mass - 6149.058906744638) / 6149.058906744638) > 1e-3):
            assert False,f'Regression failed with vessel_line_rupture_calculation.discharge_result.release_mass = {vessel_line_rupture_calculation.discharge_result.release_mass }'
        print(f'vessel_line_rupture_calculation.discharge_result.release_mass: {vessel_line_rupture_calculation.discharge_result.release_mass} [kg]')
        print(f'SUCCESS: vessel_line_rupture_calculation ({vessel_line_rupture_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED vessel_line_rupture_calculation with result code {resultCode}'



import os
import pathlib
import sys
from pickle import GET

# When running locally the environment variable PYPWS_RUN_LOCALLY needs to be set to True.
# Check if the environment variable is set
PYPWS_RUN_LOCALLY = os.getenv('PYPWS_RUN_LOCALLY')
if PYPWS_RUN_LOCALLY and PYPWS_RUN_LOCALLY.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')


from pypws.calculations import (
    SetPhaseToReleaseForLeakScenarioCalculation,
    SetPhaseToReleaseForLineRuptureScenarioCalculation,
)
from pypws.entities import Material, MaterialComponent, State, Vessel
from pypws.enums import Phase, ResultCode, VesselConditions, VesselShape


def test_case_151():
    
    """
    Test for set phase to release for line rupture scenario calculation with the following inputs

        material_name = 'N-PENTANE+METHANE'
        state_temperature = 280.0
        state_pressure = 8.0E+05
        vessel_shape = VesselShape.VESSEL_SPHERE
        vessel_diameter = 3.0
        
    """

    # Define the material.
    material = Material(
        name="N-PENTANE+METHANE",
        components=[
            MaterialComponent(
                name="N-PENTANE",
                mole_fraction=0.5
            ),
            MaterialComponent(
                name="METHANE",
                mole_fraction=0.5
            )
        ],
        component_count = 2
    )

    # Define the properties.
    state_temperature = 280.0
    state_pressure = 8.0E+06
    vessel_shape = VesselShape.VESSEL_SPHERE
    vessel_diameter = 5.0

    # Create a vessel.
    vessel = Vessel(state = State(pressure= state_pressure, temperature = state_temperature, liquid_fraction = 0.8), material = material, diameter = vessel_diameter, shape = vessel_shape, vessel_conditions = VesselConditions.STRATIFIED_TWO_PHASE_VESSEL, liquid_fill_fraction_by_volume = 0.6)

    # Create a set phase to be released for line rupture calculation.
    set_phase_to_be_released_for_line_rupture_scenario_calculation = SetPhaseToReleaseForLineRuptureScenarioCalculation(vessel = vessel, release_elevation = 1.0, phase_to_release = Phase.LIQUID)

    # Run the calculation.
    print ('Running set_phase_to_be_released_for_line_rupture_scenario_calculation')
    result_code = set_phase_to_be_released_for_line_rupture_scenario_calculation.run()

    # Print any messages.
    if len(set_phase_to_be_released_for_line_rupture_scenario_calculation.messages) > 0:
        print('Messages:')
        for message in set_phase_to_be_released_for_line_rupture_scenario_calculation.messages:
            print(message)

    if result_code == ResultCode.SUCCESS:
        if (set_phase_to_be_released_for_line_rupture_scenario_calculation.phase_to_release != 3):
            assert False, f'Regression failed with set_phase_to_be_released_for_line_rupture_scenario_calculation with phase_to_release = {set_phase_to_be_released_for_line_rupture_scenario_calculation.phase_to_release}'
        if (abs((set_phase_to_be_released_for_line_rupture_scenario_calculation.pipe_height_fraction_updated - 0.5668689228522562)/0.5668689228522562)> 1e-3):
            assert False,f'Regression failed with set_phase_to_be_released_for_line_rupture_scenario_calculation with pipe_height_fraction_updated = {set_phase_to_be_released_for_line_rupture_scenario_calculation.pipe_height_fraction_updated}'
        print(f'SUCCESS: set_phase_to_be_released_for_line_rupture_scenario_calculation ({set_phase_to_be_released_for_line_rupture_scenario_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED set_phase_to_be_released_for_line_rupture_scenario_calculation with result code {result_code}'


import os
import pathlib
import sys

run_locally = os.getenv('PYPWS_RUN_LOCALLY')
if run_locally and run_locally.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')

from pypws.calculations import LoadMassInventoryVesselForLineRuptureScenarioCalculation, VesselLineRuptureLinkedRunCalculation
from pypws.entities import Material, MaterialComponent, Substrate, Weather, DispersionParameters, DispersionOutputConfig, FlammableParameters, FlammableOutputConfig, ExplosionParameters, ExplosionOutputConfig, ExplosionConfinedVolume, DischargeParameters, Bund
from pypws.enums import ResultCode, AtmosphericStabilityClass, WindProfileFlag, SpecialConcentration


def test_case_163():

    material = Material("N-HEXANE", [MaterialComponent("N-HEXANE", 1.0)], component_count = 1)


    # Create a load mass inventory vessel for line rupture scenario calculation using the material.
    load_mass_inventory_vessel_for_line_rupture_scenario_calculation = LoadMassInventoryVesselForLineRuptureScenarioCalculation(material = material, temperature = 250, pressure = float(7e5), mass = float(9876), pipe_length = 7.5, pipe_diameter = 0.1, release_elevation = 0.0, release_angle = 0.0)

    # Run the calculation
    print('Running load_mass_inventory_vessel_for_line_rupture_scenario_calculation...')
    resultCode = load_mass_inventory_vessel_for_line_rupture_scenario_calculation.run()

    # Print any messages.
    if len(load_mass_inventory_vessel_for_line_rupture_scenario_calculation.messages) > 0:
        print('Messages:')
        for message in load_mass_inventory_vessel_for_line_rupture_scenario_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        if (abs((load_mass_inventory_vessel_for_line_rupture_scenario_calculation.vessel.diameter-3.0323298714998153)/3.0323298714998153)>1e-3):
            assert False,f'Regression failed with load_mass_inventory_vessel_for_line_rupture_scenario_calculation.vessel.diameter = {load_mass_inventory_vessel_for_line_rupture_scenario_calculation.vessel.diameter}'
        if (abs((load_mass_inventory_vessel_for_line_rupture_scenario_calculation.vessel.location.z-(-2.7170445294165657))/(-2.7170445294165657))>1e-3):
            assert False,f'Regression failed with load_mass_inventory_vessel_for_line_rupture_scenario_calculation.vessel.location.z = {load_mass_inventory_vessel_for_line_rupture_scenario_calculation.vessel.location.z}'
        if (abs((load_mass_inventory_vessel_for_line_rupture_scenario_calculation.line_rupture.pipe_diameter-0.1)/0.1)>1e-3):
            assert False,f'Regression failed with load_mass_inventory_vessel_for_line_rupture_scenario_calculation.line_rupture.pipe_diameter = {load_mass_inventory_vessel_for_line_rupture_scenario_calculation.line_rupture.pipe_diameter}'
        if (abs((load_mass_inventory_vessel_for_line_rupture_scenario_calculation.line_rupture.pipe_height_fraction-0.8960253813259087)/0.8960253813259087)>1e-3):
            assert False,f'Regression failed with load_mass_inventory_vessel_for_line_rupture_scenario_calculation.line_rupture.pipe_height_fraction = {load_mass_inventory_vessel_for_line_rupture_scenario_calculation.line_rupture.pipe_height_fraction}'
        print(f'SUCCESS: load_mass_inventory_vessel_for_line_rupture_scenario_calculation ({load_mass_inventory_vessel_for_line_rupture_scenario_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED load_mass_inventory_vessel_for_line_rupture_scenario_calculation with result code {resultCode}'

    # Define the weather conditions
    weather = Weather( wind_speed = 2.0, stability_class = AtmosphericStabilityClass.STABILITY_CD, wind_profile_flag = WindProfileFlag.LOGARITHMIC_PROFILE)

    # Define the substrate
    substrate = Substrate(bund = Bund(specify_bund=True, bund_height = 2.0, bund_diameter = 20.0))

    # Define the dispersion parameters
    dispersion_parameters = [DispersionParameters(averaging_time = 18.75, lfl_fraction_to_stop=0.15), DispersionParameters(averaging_time = 18.75)]

    # Define the dispersion output configuration
    dispersion_output_configs_flammable = [DispersionOutputConfig(special_concentration = SpecialConcentration.LFL_FRACTION, lfl_fraction_value= 0.15,  elevation = 0.0)]
    dispersion_output_configs_toxic = [DispersionOutputConfig(special_concentration = SpecialConcentration.NOT_DEFINED, concentration = 5e-5, elevation = 0.0)]

    # Define the flammable parameters
    flammable_parameters = FlammableParameters()

    # Define the flammable output configuration
    flammable_output_configs = [FlammableOutputConfig()]

    # Define the explosion parameters
    explosion_parameters = ExplosionParameters()

    # Define the explosion output configuration
    explosion_output_configs = [ExplosionOutputConfig()]

    # Define the explosion confined volumes
    explosion_confined_volumes = [ExplosionConfinedVolume()]

    # 
    vessel_line_rupture_linked_run_calculation = VesselLineRuptureLinkedRunCalculation(
        vessel = load_mass_inventory_vessel_for_line_rupture_scenario_calculation.vessel,
        line_rupture = load_mass_inventory_vessel_for_line_rupture_scenario_calculation.line_rupture,
        discharge_parameters = DischargeParameters(),
        substrate = substrate,
        weather = weather,
        dispersion_parameters = dispersion_parameters,
        dispersion_parameter_count = len(dispersion_parameters),
        end_point_concentration = 0.0,
        flammable_parameters = flammable_parameters,
        explosion_parameters = explosion_parameters,
        dispersion_flam_output_configs = dispersion_output_configs_flammable,
        dispersion_flam_output_config_count = len(dispersion_output_configs_flammable),
        dispersion_toxic_output_configs = dispersion_output_configs_toxic,
        dispersion_toxic_output_config_count = len(dispersion_output_configs_toxic),
        flammable_output_configs = flammable_output_configs,
        flammable_output_config_count = len(flammable_output_configs),
        explosion_output_configs = explosion_output_configs,
        explosion_output_config_count = len(explosion_output_configs),
        explosion_confined_volumes = explosion_confined_volumes,
        explosion_confined_volume_count = len(explosion_confined_volumes)
    )

    # Run the calculation
    print('Running vessel_line_rupture_linked_run_calculation...')
    resultCode = vessel_line_rupture_linked_run_calculation.run()

    # Print any messages.
    if len(vessel_line_rupture_linked_run_calculation.messages) > 0:
        print('Messages:')
        for message in vessel_line_rupture_linked_run_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        if (abs(vessel_line_rupture_linked_run_calculation.discharge_record.mass_flow - 131.73664600034286)/131.73664600034286 > 1e-3):
            assert False,f'Regression failed with vessel_line_rupture_linked_run_calculation.discharge_record.mass_flow = {vessel_line_rupture_linked_run_calculation.discharge_record.mass_flow}'
        if (vessel_line_rupture_linked_run_calculation.jet_fire_flame_result.flame_length != 0.0):
            assert False,f'Regression failed with vessel_line_rupture_linked_run_calculation.jet_fire_flame_result.flame_length = {vessel_line_rupture_linked_run_calculation.jet_fire_flame_result.flame_length}'
        if (abs(vessel_line_rupture_linked_run_calculation.pool_fire_flame_result.flame_diameter - 20.00299644470215)/20.00299644470215 > 1e-3):
            assert False,f'Regression failed with vessel_line_rupture_linked_run_calculation.pool_fire_flame_result.flame_diameter = {vessel_line_rupture_linked_run_calculation.pool_fire_flame_result.flame_diameter}'
        if (len(vessel_line_rupture_linked_run_calculation.flam_conc_contour_points) != 563):
            assert False,f'Regression failed with len(vessel_line_rupture_linked_run_calculation.flam_conc_contour_points) = {len(vessel_line_rupture_linked_run_calculation.flam_conc_contour_points)}'
        if (len(vessel_line_rupture_linked_run_calculation.toxic_conc_contour_points) != 623):
            assert False,f'Regression failed with len(vessel_line_rupture_linked_run_calculation.toxic_conc_contour_points) = {len(vessel_line_rupture_linked_run_calculation.toxic_conc_contour_points)}'
        if (abs(vessel_line_rupture_linked_run_calculation.area_footprint_flam_conc[0]- 585.0061568126835)/ 585.0061568126835 > 1e-3):
            assert False,f'Regression failed with vessel_line_rupture_linked_run_calculation.area_footprint_flam_conc[0] = {vessel_line_rupture_linked_run_calculation.area_footprint_flam_conc[0]}'
        if (abs(vessel_line_rupture_linked_run_calculation.area_footprint_toxic_conc[0] -12934.3464614692)/12934.3464614692 > 1e-3):
            assert False,f'Regression failed with vessel_line_rupture_linked_run_calculation.area_footprint_toxic_conc[0] = {vessel_line_rupture_linked_run_calculation.area_footprint_toxic_conc[0]}'
        if (vessel_line_rupture_linked_run_calculation.area_contour_jet[0]!= 0.0):
            assert False,f'Regression failed with vessel_line_rupture_linked_run_calculation.area_contour_jet[0] = {vessel_line_rupture_linked_run_calculation.area_contour_jet[0]}'
        if (abs(vessel_line_rupture_linked_run_calculation.area_contour_pool[0] - 4498.14606654948)/4498.14606654948 > 1e-3):
            assert False,f'Regression failed with vessel_line_rupture_linked_run_calculation.area_contour_pool[0] = {vessel_line_rupture_linked_run_calculation.area_contour_pool[0]}'
        if (vessel_line_rupture_linked_run_calculation.explosion_overpressure_results[0].exploded_mass!= 0.0):
            assert False,f'Regression failed with vessel_line_rupture_linked_run_calculation.explosion_overpressure_results[0].exploded_mass = {vessel_line_rupture_linked_run_calculation.explosion_overpressure_results[0].exploded_mass}'
        if (vessel_line_rupture_linked_run_calculation.explosion_overpressure_results[0].maximum_distance != 0.0):
            assert False,f'Regression failed with vessel_line_rupture_linked_run_calculation.explosion_overpressure_results[0].maximum_distance = {vessel_line_rupture_linked_run_calculation.explosion_overpressure_results[0].maximum_distance}'
        print(f'Mass flow ={vessel_line_rupture_linked_run_calculation.discharge_record.mass_flow}')
        print(f'Flame lenght = {vessel_line_rupture_linked_run_calculation.jet_fire_flame_result.flame_length}')
        print(f'Flame diameter = {vessel_line_rupture_linked_run_calculation.pool_fire_flame_result.flame_diameter}')
        print(f'Length of the dispersion flammable contour array = {len(vessel_line_rupture_linked_run_calculation.flam_conc_contour_points)}')
        print(f'Length of the dispersion toxic contour array = {len(vessel_line_rupture_linked_run_calculation.toxic_conc_contour_points)}')
        print(f'Area of the flammable cloud = {vessel_line_rupture_linked_run_calculation.area_footprint_flam_conc[0]}')
        print(f'Area of the toxic cloud = {vessel_line_rupture_linked_run_calculation.area_footprint_toxic_conc[0]}')
        print(f'Area of the jet fire = {vessel_line_rupture_linked_run_calculation.area_contour_jet[0]}')
        print(f'Area of the pool = {vessel_line_rupture_linked_run_calculation.area_contour_pool[0]}')
        print(f'Flammable mass = {vessel_line_rupture_linked_run_calculation.explosion_overpressure_results[0].exploded_mass}')
        print(f'Explosion maximum distance = {vessel_line_rupture_linked_run_calculation.explosion_overpressure_results[0].maximum_distance}')
        print(f'SUCCESS: vessel_line_rupture_linked_run_calculation ({vessel_line_rupture_linked_run_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED vessel_line_rupture_linked_run_calculation with result code {resultCode}'		



import os
import pathlib
import sys

# When running locally the environment variable PYPWS_RUN_LOCALLY needs to be set to True.
# Check if the environment variable is set
if os.getenv('PYPWS_RUN_LOCALLY') != None and os.getenv('PYPWS_RUN_LOCALLY').lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')

from pypws.calculations import (
    VesselLeakMaxFlammableCloudCalculation,
    VesselStateCalculation,
)
from pypws.entities import (
    DischargeParameters,
    DispersionOutputConfig,
    DispersionParameters,
    Leak,
    Material,
    MaterialComponent,
    State,
    Substrate,
    Vessel,
    Weather,
)
from pypws.enums import (
    AtmosphericStabilityClass,
    Resolution,
    ResultCode,
    SpecialConcentration,
    SurfaceType,
    TimeVaryingOption,
    VesselShape,
    WindProfileFlag
)

"""
This sample demonstrates how to use the vessel leak maximum flammable cloud calculation along with with the dependent entities.
"""

def test_case_94():

    # Set the case properties.
    material_name = 'N-PENTANE'
    state_temperature = 280.0
    state_pressure = 1.50E+05
    vessel_shape = VesselShape.HORIZONTAL_CYLINDER
    vessel_length = 5.0
    vessel_diameter = 2.0
    leak_hole_diameter = 0.05
    time_varying_option = TimeVaryingOption.TIME_VARYING_RATE
    leak_hole_height_fraction = 0.0

    # Define the initial state of the vessel.
    state = State(temperature = state_temperature, pressure = state_pressure, liquid_fraction = 0.0)

    # Define the material contained by the vessel.
    material = Material(material_name, [MaterialComponent(material_name, 1.0)])

    # Create a vessel state calculation using the material and state.
    vessel_state_calculation = VesselStateCalculation(material, state)

    # Run the vessel state calculation.
    print('Running vessel_state_calculation...')
    resultCode = vessel_state_calculation.run()

    # Print any messages.
    if len(vessel_state_calculation.messages) > 0:
        print('Messages:')
        for message in vessel_state_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        print(f'SUCCESS: vessel_state_calculation ({vessel_state_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED vessel_state_calculation with result code {resultCode}'

    # Create a vessel to use in the leak calculation using the previously defined entities.
    vessel = Vessel(state = vessel_state_calculation.output_state, material = vessel_state_calculation.material, vessel_conditions = vessel_state_calculation.vessel_conditions, diameter = vessel_diameter, length = vessel_length, shape = vessel_shape, liquid_fill_fraction_by_volume = 0.5)

    # Create a leak to use in the vessel leak calculation.
    # The leak has a hole of diameter of 0.05m.  The time varying option is set topytest initial rate.
    leak = Leak(hole_diameter = leak_hole_diameter, hole_height_fraction = leak_hole_height_fraction , time_varying_option = time_varying_option)

    # Create discharge parameters to use in the vessel leak calculation taking all the default values.
    discharge_parameters = DischargeParameters()

    # Define the weather
    weather = Weather(wind_speed = 5.0, stability_class = AtmosphericStabilityClass.STABILITY_D, wind_profile_flag = WindProfileFlag.LOGARITHMIC_PROFILE)

    # Define the substrate
    substrate = Substrate(surface_roughness = 0.2, surface_type = SurfaceType.LAND)

    # Define the dispersion parameters
    dispersion_parameters = DispersionParameters()

    # Define the dispersion output configuration
    dispersion_output_config = DispersionOutputConfig(special_concentration = SpecialConcentration.LFL_FRACTION, resolution = Resolution.HIGH, elevation=0.0)

    # Create the vessel leak maximum flammable cloud calculation using the previously defined entities.
    vessel_leak_max_flammable_cloud_calculation = VesselLeakMaxFlammableCloudCalculation(vessel=vessel, leak=leak, discharge_parameters=discharge_parameters, weather=weather, substrate=substrate, dispersion_parameters=dispersion_parameters, dispersion_output_config=dispersion_output_config)

    # Run the calculation.
    print('Running vessel_leak_max_flammable_cloud_calculation...')
    resultCode = vessel_leak_max_flammable_cloud_calculation.run()

    # Print any messages.
    if len(vessel_leak_max_flammable_cloud_calculation.messages) > 0:
        print('Messages:')
        for message in vessel_leak_max_flammable_cloud_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        if (vessel_leak_max_flammable_cloud_calculation.vessel_leak_max_flammable_cloud_results.phase != 3):
            assert False,f'Regression failed with vessel_leak_max_flammable_cloud_calculation.vessel_leak_max_flammable_cloud_results.phase = {vessel_leak_max_flammable_cloud_calculation.vessel_leak_max_flammable_cloud_results.phase}'
        if (abs((vessel_leak_max_flammable_cloud_calculation.vessel_leak_max_flammable_cloud_results.lfl_extent- 15.196817260385107)/ 15.196817260385107)>1e-3):
            assert False,f'Regression failed with vessel_leak_max_flammable_cloud_calculation.vessel_leak_max_flammable_cloud_results.lfl_extent = {vessel_leak_max_flammable_cloud_calculation.vessel_leak_max_flammable_cloud_results.lfl_extent}'
        if (abs((vessel_leak_max_flammable_cloud_calculation.vessel_leak_max_flammable_cloud_results.lfl_area- 407.2282363319728)/ 407.2282363319728)>1e-3):
            assert False,f'Regression failed with vessel_leak_max_flammable_cloud_calculation.vessel_leak_max_flammable_cloud_results.lfl_area = {vessel_leak_max_flammable_cloud_calculation.vessel_leak_max_flammable_cloud_results.lfl_area}'
        if (abs((vessel_leak_max_flammable_cloud_calculation.vessel_leak_max_flammable_cloud_results.lfl_height-  0.3118404430192289)/ 0.3118404430192289)>1e-3):
            assert False,f'Regression failed with vessel_leak_max_flammable_cloud_calculation.vessel_leak_max_flammable_cloud_results.lfl_height = {vessel_leak_max_flammable_cloud_calculation.vessel_leak_max_flammable_cloud_results.lfl_height}'
        print(f'SUCCESS: vessel_leak_max_flammable_cloud_calculation ({vessel_leak_max_flammable_cloud_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED vessel_leak_max_flammable_cloud_calculation with result code {resultCode}'


import os
import pathlib
import sys

run_locally = os.getenv('PYPWS_RUN_LOCALLY')
if run_locally and run_locally.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')

from pypws.calculations import LoadMassInventoryVesselForReliefValveScenarioCalculation, VesselReliefValveLinkedRunCalculation
from pypws.entities import Material, MaterialComponent, Weather, Substrate, DispersionParameters, FlammableParameters, ExplosionParameters, ExplosionConfinedVolume, DischargeParameters, DispersionOutputConfig, FlammableOutputConfig, ExplosionOutputConfig
from pypws.enums import ResultCode, AtmosphericStabilityClass, WindProfileFlag, SpecialConcentration


def test_case_165():

    material = Material("PROPANE", [MaterialComponent("PROPANE", 1.0)], component_count = 1)


    # Create a load mass inventory vessel for relief valve scenario calculation using the material.
    load_mass_inventory_vessel_for_relief_valve_scenario_calculation = LoadMassInventoryVesselForReliefValveScenarioCalculation(material = material, temperature = 350, pressure = float(11e5), mass = float(1e3), pipe_length = 10.0, pipe_diameter = 0.5, constriction_size = 0.15, release_elevation = 1.0, release_angle = 0.35)

    # Run the calculation
    print('Running load_mass_inventory_vessel_for_relief_valve_scenario_calculation...')
    resultCode = load_mass_inventory_vessel_for_relief_valve_scenario_calculation.run()

    # Print any messages.
    if len(load_mass_inventory_vessel_for_relief_valve_scenario_calculation.messages) > 0:
        print('Messages:')
        for message in load_mass_inventory_vessel_for_relief_valve_scenario_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        if (abs((load_mass_inventory_vessel_for_relief_valve_scenario_calculation.vessel.diameter-4.665573040127879)/4.665573040127879)>1e-3):
            assert False, f'Regression failed with load_mass_inventory_vessel_for_relief_valve_scenario_calculation.vessel.diameter = {load_mass_inventory_vessel_for_relief_valve_scenario_calculation.vessel.diameter}'
        if (abs((load_mass_inventory_vessel_for_relief_valve_scenario_calculation.vessel.location.z-(-1.3327865200639395))/(-1.3327865200639395))>1e-3):
            assert False, f'Regression failed load_mass_inventory_vessel_for_relief_valve_scenario_calculation.vessel.location.z = {load_mass_inventory_vessel_for_relief_valve_scenario_calculation.vessel.location.z}'
        if (abs((load_mass_inventory_vessel_for_relief_valve_scenario_calculation.relief_valve.pipe_diameter-0.5)/0.5)>1e-3):
            assert False, f'Regression failed with load_mass_inventory_vessel_for_relief_valve_scenario_calculation.relief_valve.pipe_diameter = {load_mass_inventory_vessel_for_relief_valve_scenario_calculation.relief_valve.pipe_diameter}'
        if (abs((load_mass_inventory_vessel_for_relief_valve_scenario_calculation.relief_valve.pipe_height_fraction-0.5)/0.5)>1e-3):
            assert False, f'Regression failed with load_mass_inventory_vessel_for_relief_valve_scenario_calculation.relief_valve.pipe_height_fraction = {load_mass_inventory_vessel_for_relief_valve_scenario_calculation.relief_valve.pipe_height_fraction}'
        print(f'SUCCESS: load_mass_inventory_vessel_for_relief_valve_scenario_calculation ({load_mass_inventory_vessel_for_relief_valve_scenario_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED load_mass_inventory_vessel_for_relief_valve_scenario_calculation with result code {resultCode}'

    # Define the weather conditions
    weather = Weather( wind_speed = 10.0, stability_class = AtmosphericStabilityClass.STABILITY_A, wind_profile_flag = WindProfileFlag.LOGARITHMIC_PROFILE)

    # Define the substrate
    substrate = Substrate()

    # Define the dispersion parameters
    dispersion_parameters = [DispersionParameters(averaging_time = 18.75), DispersionParameters(averaging_time =18.75)]

    # Define the dispersion output configuration
    dispersion_output_configs_flammable = [DispersionOutputConfig(special_concentration = SpecialConcentration.LFL_FRACTION,  elevation = 0.0)]
    dispersion_output_configs_toxic = [DispersionOutputConfig(special_concentration = SpecialConcentration.NOT_DEFINED, concentration = 5e-5, elevation = 0.0)]

    # Define the flammable parameters
    flammable_parameters = FlammableParameters()

    # Define the flammable output configuration
    flammable_output_configs = [FlammableOutputConfig()]

    # Define the explosion parameters
    explosion_parameters = ExplosionParameters()

    # Define the explosion output configuration
    explosion_output_configs = [ExplosionOutputConfig()]

    # Define the explosion confined volumes
    explosion_confined_volumes = [ExplosionConfinedVolume()]

    # 
    vessel_relief_valve_linked_run_calculation = VesselReliefValveLinkedRunCalculation(
        vessel = load_mass_inventory_vessel_for_relief_valve_scenario_calculation.vessel,
        relief_valve = load_mass_inventory_vessel_for_relief_valve_scenario_calculation.relief_valve,
        discharge_parameters = DischargeParameters(),
        substrate = substrate,
        weather = weather,
        dispersion_parameters = dispersion_parameters,
        dispersion_parameter_count = len(dispersion_parameters),
        end_point_concentration = 0.0,
        flammable_parameters = flammable_parameters,
        explosion_parameters = explosion_parameters,
        dispersion_flam_output_configs = dispersion_output_configs_flammable,
        dispersion_flam_output_config_count = len(dispersion_output_configs_flammable),
        dispersion_toxic_output_configs = dispersion_output_configs_toxic,
        dispersion_toxic_output_config_count = len(dispersion_output_configs_toxic),
        flammable_output_configs = flammable_output_configs,
        flammable_output_config_count = len(flammable_output_configs),
        explosion_output_configs = explosion_output_configs,
        explosion_output_config_count = len(explosion_output_configs),
        explosion_confined_volumes = explosion_confined_volumes,
        explosion_confined_volume_count = len(explosion_confined_volumes)
    )

    # Run the calculation
    print('Running vessel_relief_valve_linked_run_calculation...')
    resultCode = vessel_relief_valve_linked_run_calculation.run()

    # Print any messages.
    if len(vessel_relief_valve_linked_run_calculation.messages) > 0:
        print('Messages:')
        for message in vessel_relief_valve_linked_run_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        if (abs(vessel_relief_valve_linked_run_calculation.discharge_record.mass_flow - 52.423581478093844)/52.423581478093844> 1e-3):
            assert False,f'Regression failed with vessel_relief_valve_linked_run_calculation.discharge_record.mass_flow = {vessel_relief_valve_linked_run_calculation.discharge_record.mass_flow}'
        if (abs(vessel_relief_valve_linked_run_calculation.jet_fire_flame_result.flame_length - 67.47188617179833)/67.47188617179833 > 1e-3):
            assert False,f'Regression failed with vessel_relief_valve_linked_run_calculation.jet_fire_flame_result.flame_length = {vessel_relief_valve_linked_run_calculation.jet_fire_flame_result.flame_length}'
        if (vessel_relief_valve_linked_run_calculation.pool_fire_flame_result.flame_diameter != 0.0):
            assert False,f'Regression failed with vessel_relief_valve_linked_run_calculation.pool_fire_flame_result.flame_diameter = {vessel_relief_valve_linked_run_calculation.pool_fire_flame_result.flame_diameter}'
        if (len(vessel_relief_valve_linked_run_calculation.flam_conc_contour_points) != 0):
            assert False,f'Regression failed with len(vessel_relief_valve_linked_run_calculation.flam_conc_contour_points) = {len(vessel_relief_valve_linked_run_calculation.flam_conc_contour_points)}'
        if (len(vessel_relief_valve_linked_run_calculation.toxic_conc_contour_points) != 629):
            assert False,f'Regression failed with len(vessel_relief_valve_linked_run_calculation.toxic_conc_contour_points) = {len(vessel_relief_valve_linked_run_calculation.toxic_conc_contour_points)}'
        if (vessel_relief_valve_linked_run_calculation.area_footprint_flam_conc[0]!= 0.0):
            assert False,f'Regression failed with vessel_relief_valve_linked_run_calculation.area_footprint_flam_conc[0] = {vessel_relief_valve_linked_run_calculation.area_footprint_flam_conc[0]}'
        if (abs(vessel_relief_valve_linked_run_calculation.area_footprint_toxic_conc[0] -81936.99336133621)/81936.99336133621 > 1e-3):
            assert False,f'Regression failed with vessel_relief_valve_linked_run_calculation.area_footprint_toxic_conc[0] = {vessel_relief_valve_linked_run_calculation.area_footprint_toxic_conc[0]}'
        if (abs(vessel_relief_valve_linked_run_calculation.area_contour_jet[0]-  20491.484283624566)/ 20491.484283624566 > 1e-3):
            assert False,f'Regression failed with vessel_relief_valve_linked_run_calculation.area_contour_jet[0] = {vessel_relief_valve_linked_run_calculation.area_contour_jet[0]}'
        if (vessel_relief_valve_linked_run_calculation.area_contour_pool[0] != 0.0):
            assert False,f'Regression failed with vessel_relief_valve_linked_run_calculation.area_contour_pool[0] = {vessel_relief_valve_linked_run_calculation.area_contour_pool[0]}'
        if (abs(vessel_relief_valve_linked_run_calculation.explosion_overpressure_results[0].exploded_mass-20.535225036338556)/20.535225036338556 > 1e-3):	
            assert False,f'Regression failed with vessel_relief_valve_linked_run_calculation.explosion_overpressure_results[0].exploded_mass = {vessel_relief_valve_linked_run_calculation.explosion_overpressure_results[0].exploded_mass}'
        if (abs(vessel_relief_valve_linked_run_calculation.explosion_overpressure_results[0].maximum_distance - 149.87854518675923)/149.87854518675923 > 1e-3):
            assert False,f'Regression failed with vessel_relief_valve_linked_run_calculation.explosion_overpressure_results[0].maximum_distance = {vessel_relief_valve_linked_run_calculation.explosion_overpressure_results[0].maximum_distance}'
        print(f'Mass flow ={vessel_relief_valve_linked_run_calculation.discharge_record.mass_flow}')
        print(f'Flame lenght = {vessel_relief_valve_linked_run_calculation.jet_fire_flame_result.flame_length}')
        print(f'Flame diameter = {vessel_relief_valve_linked_run_calculation.pool_fire_flame_result.flame_diameter}')
        print(f'Length of the dispersion flammable contour array = {len(vessel_relief_valve_linked_run_calculation.flam_conc_contour_points)}')
        print(f'Length of the dispersion toxic contour array = {len(vessel_relief_valve_linked_run_calculation.toxic_conc_contour_points)}')
        print(f'Area of the flammable cloud = {vessel_relief_valve_linked_run_calculation.area_footprint_flam_conc[0]}')
        print(f'Area of the toxic cloud = {vessel_relief_valve_linked_run_calculation.area_footprint_toxic_conc[0]}')
        print(f'Area of the jet fire = {vessel_relief_valve_linked_run_calculation.area_contour_jet[0]}')
        print(f'Area of the pool = {vessel_relief_valve_linked_run_calculation.area_contour_pool[0]}')
        print(f'Flammable mass = {vessel_relief_valve_linked_run_calculation.explosion_overpressure_results[0].exploded_mass}')
        print(f'Explosion maximum distance = {vessel_relief_valve_linked_run_calculation.explosion_overpressure_results[0].maximum_distance}')
        print(f'SUCCESS: vessel_relief_valve_linked_run_calculation ({vessel_relief_valve_linked_run_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED vessel_relief_valve_linked_run_calculation with result code {resultCode}'		



import os
import pathlib
import sys

run_locally = os.getenv('PYPWS_RUN_LOCALLY')
if run_locally and run_locally.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')

from pypws.calculations import (
    DispersionCalculation,
    LethalityDistanceCalculation,
    VesselCatastrophicRuptureCalculation,
    VesselStateCalculation,
)
from pypws.entities import (
    Bund,
    DischargeParameters,
    DispersionOutputConfig,
    DispersionParameters,
    Material,
    MaterialComponent,
    State,
    Substrate,
    Vessel,
    Weather,
)
from pypws.enums import AtmosphericStabilityClass, ResultCode, SurfaceType, VesselShape, WindProfileFlag


def test_case_60():

    """
    lethality distance calculation test case with the following properties:

        material_name = 'CARBON DIOXIDE (TOXIC)'
        state_temperature = 280.0
        state_pressure = 8.0E+06
        vessel_shape = VesselShape.VESSEL_SPHERE
        vessel_diameter = 5.0
        liquid_fraction = 0.8
        surface_type = SurfaceType.LAND
        surface_roughness = 0.2
        wind_speed = 5.0
        stability_class = AtmosphericStabilityClass.STABILITY_A
        end_point_concentration = 0.0
        specify_bund = True
        bund_height = 1.0
        bund_diameter = 8.0

    """

    material_name = 'CARBON DIOXIDE (TOXIC)'
    state_temperature = 280.0
    state_pressure = 8.0E+06
    liquid_fraction = 0.8
    vessel_shape = VesselShape.VESSEL_SPHERE
    vessel_diameter = 5.0
    liquid_fill_fraction_by_volume=0.8
    surface_type = SurfaceType.LAND
    surface_roughness = 0.2
    wind_speed = 5.0
    stability_class = AtmosphericStabilityClass.STABILITY_A
    wind_profile_flag = WindProfileFlag.LOGARITHMIC_PROFILE
    end_point_concentration = 0.0
    specify_bund = True
    bund_height = 1.0
    bund_diameter = 8.0

    material = Material(material_name, [MaterialComponent(material_name, 1.0)], component_count = 1)
    state = State(temperature=state_temperature, pressure= state_pressure, liquid_fraction=liquid_fraction)

    # Create a vessel state calculation using the material and state.
    vessel_state_calculation = VesselStateCalculation(material, state)

    # Run the vessel state calculation.
    print('Running vessel_state_calculation...')
    resultCode = vessel_state_calculation.run()

    # Print any messages.
    if len(vessel_state_calculation.messages) > 0:
        print('Messages:')
        for message in vessel_state_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        print(f'SUCCESS: vessel_state_calculation ({vessel_state_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED vessel_state_calculation with result code {resultCode}'

    vessel = Vessel(state=state,
                    material=material,
                    liquid_fill_fraction_by_volume=liquid_fill_fraction_by_volume,
                    shape=vessel_shape,
                    diameter=vessel_diameter,
                    vessel_conditions = vessel_state_calculation.vessel_conditions
                    )

    discharge_parameters = DischargeParameters()

    vessel_catastrophic_rupture_calculation = VesselCatastrophicRuptureCalculation(vessel, discharge_parameters)

    print('Running vessel_catastrophic_rupture_calculation...')
    resultCode = vessel_catastrophic_rupture_calculation.run()

    # Print any messages.
    if len(vessel_catastrophic_rupture_calculation.messages) > 0:
        print('Messages:')
        for message in vessel_catastrophic_rupture_calculation.messages:
            print(message)

    if resultCode == resultCode.SUCCESS:
        print(f'SUCCESS: vessel_catastrophic_rupture_calculation ({vessel_catastrophic_rupture_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED vessel_catastrophic_rupture_calculation with result code {resultCode}'

    # Set up the entities required by the dispersion calculation.
    bund = Bund(specify_bund=specify_bund, bund_height=bund_height, bund_diameter=bund_diameter)
    substrate = Substrate(bund=bund, surface_type=surface_type, surface_roughness=surface_roughness)
    weather = Weather(wind_speed=wind_speed, stability_class=stability_class, wind_profile_flag = WindProfileFlag.LOGARITHMIC_PROFILE)
    dispersion_parameters = DispersionParameters()

    # Set up the dispersion calculation.
    dispersion_calculation = DispersionCalculation(vessel_catastrophic_rupture_calculation.exit_material, substrate, vessel_catastrophic_rupture_calculation.discharge_result, vessel_catastrophic_rupture_calculation.discharge_records, len(vessel_catastrophic_rupture_calculation.discharge_records), weather, dispersion_parameters, end_point_concentration)

    print('Running dispersion_calculation...')
    resultCode = dispersion_calculation.run()

    # Print any messages.
    if len(dispersion_calculation.messages) > 0:
        print('Messages:')
        for message in dispersion_calculation.messages:
            print(message)

    if resultCode == resultCode.SUCCESS:
        print(f'SUCCESS: dispersion_calculation ({dispersion_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED dispersion_calculation with result code {resultCode}'

    # Set up the entities required by the lethality distance calculation.
    dispersion_output_config = DispersionOutputConfig(concentration = end_point_concentration)

    # Set up the lethality distance calculation.
    lethality_distance_calculation = LethalityDistanceCalculation(scalar_udm_outputs= dispersion_calculation.scalar_udm_outputs, weather= weather, dispersion_records= dispersion_calculation.dispersion_records, dispersion_record_count= len(dispersion_calculation.dispersion_records), substrate=substrate, dispersion_output_config=dispersion_output_config, material=vessel_catastrophic_rupture_calculation.exit_material, dispersion_parameters=dispersion_parameters)

    print('Running lethality_distance_calculation...')
    resultCode = lethality_distance_calculation.run()

    # Print any messages.
    if len(lethality_distance_calculation.messages) > 0:
        print('Messages:')
        for message in lethality_distance_calculation.messages:
            print(message)

    if resultCode == resultCode.SUCCESS:
        if (len(lethality_distance_calculation.toxic_records)!=1830):
            assert False,f'Regression failed with len(lethality_distance_calculation.toxic_records) = {len(lethality_distance_calculation.toxic_records)}'
        if (len(lethality_distance_calculation.toxic_records)>0):
            if (abs((lethality_distance_calculation.toxic_records[len(lethality_distance_calculation.toxic_records)-1].toxic_result-0.0004837476263379603)/0.0004837476263379603)>1e-3):
                assert False,f'Regression failed with last element of toxic record = {lethality_distance_calculation.toxic_records[len(lethality_distance_calculation.toxic_records)-1].toxic_result}'
        print(f'SUCCESS: lethality_distance_calculation ({lethality_distance_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED lethality_distance_calculation with result code {resultCode}'



import os
import pathlib
import sys
from pickle import GET

# When running locally the environment variable PYPWS_RUN_LOCALLY needs to be set to True.
# Check if the environment variable is set
PYPWS_RUN_LOCALLY = os.getenv('PYPWS_RUN_LOCALLY')
if PYPWS_RUN_LOCALLY and PYPWS_RUN_LOCALLY.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')


from pypws.materials import get_material_by_cas_id
from pypws.entities import Material
from pypws.utilities import where

def test_get_material_by_casid_1_2_PROPYLENE_OXIDE():

    """
    Test to get the material by CASID for 1,2-PROPYLENE OXIDE.
        
    """

    # Invoke the method.
    print ('Running get_material_by_cas_id')
    materials =get_material_by_cas_id(75569)

    # Assert that the material is not None and that 1,2-PROPYLENE OXIDE has been returned..
    assert materials is not None, 'No materials returned'
    assert len(materials) > 0, 'A material matching the CAS ID was not returned'
    assert where(materials, lambda x: x.name == '1,2-PROPYLENE OXIDE') is not None, '1,2-PROPYLENE OXIDE not returned'

    for material in materials:
        print ('Material:', material)




import os
import pathlib
import sys

# When running locally the environment variable PYPWS_RUN_LOCALLY needs to be set to True.
# Check if the environment variable is set
if os.getenv('PYPWS_RUN_LOCALLY') != None and os.getenv('PYPWS_RUN_LOCALLY').lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')

from pypws.calculations import VesselLeakCalculation, VesselStateCalculation
from pypws.entities import (
    DischargeParameters,
    Leak,
    LocalPosition,
    Material,
    MaterialComponent,
    State,
    Vessel,
)
from pypws.enums import ResultCode, TimeVaryingOption, VesselShape

"""
This sample demonstrates how to use the vessel leak calculation along with with the dependent entities.
"""

def test_case_10():

    """
    Vessel leak calculation test case with the following properties:

        material_name = 'N-BUTANE'
        state_temperature = 300.0
        state_pressure = 3.0E+05
        vessel_shape = VesselShape.VESSEL_CUBOID
        vessel_height = 2.0
        vessel_width = 1.0
        vessel_length = 3.0
        leak_hole_diameter = 0.05
        time_varying_option = TimeVaryingOption.INITIAL_RATE
        leak_hole_height_fraction = 0.2
    """

    # Set the case properties.
    material_name = 'N-BUTANE'
    state_temperature = 300.0
    state_pressure = 3.0E+05
    vessel_shape = VesselShape.VESSEL_CUBOID
    vessel_height = 2.0
    vessel_width = 1.0
    vessel_length = 3.0
    leak_hole_diameter = 0.05
    time_varying_option = TimeVaryingOption.INITIAL_RATE
    leak_hole_height_fraction = 0.2

    # Define the initial state of the vessel.
    state = State(temperature = state_temperature, pressure = state_pressure, liquid_fraction = 0.0)

    # Define the material contained by the vessel.
    material = Material(material_name, [MaterialComponent(material_name, 1.0)])

    # Create a vessel state calculation using the material and state.
    vessel_state_calculation = VesselStateCalculation(material, state)

    # Run the vessel state calculation.
    print('Running vessel_state_calculation...')
    resultCode = vessel_state_calculation.run()

    # Print any messages.
    if len(vessel_state_calculation.messages) > 0:
        print('Messages:')
        for message in vessel_state_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        print(f'SUCCESS: vessel_state_calculation ({vessel_state_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED vessel_state_calculation with result code {resultCode}'

    # Create a vessel to use in the leak calculation using the previously defined entities.
    # The vessel is a horizontal cylinder with a diameter of 8m and a length of 16m.
    # All other values are defaulted.
    vessel = Vessel(state = vessel_state_calculation.output_state, material = vessel_state_calculation.material, vessel_conditions = vessel_state_calculation.vessel_conditions, height = vessel_height, width = vessel_width, length = vessel_length, shape = vessel_shape, liquid_fill_fraction_by_volume = 0.5)

    # Create a leak to use in the vessel leak calculation.
    # The leak has a hole of diameter of 0.05m.  The time varying option is set topytest initial rate.
    leak = Leak(hole_diameter = leak_hole_diameter, hole_height_fraction = leak_hole_height_fraction , time_varying_option = time_varying_option)

    # Create discharge parameters to use in the vessel leak calculation taking all the default values.
    discharge_parameters = DischargeParameters()

    # Create a vessel leak calculation using the vessel, leak, and discharge parameters.
    vessel_leak_calculation = VesselLeakCalculation(vessel, leak, discharge_parameters)

    # Run the vessel leak calculation.
    print('Running vessel_leak_calculation...')
    resultCode = vessel_leak_calculation.run()

    # Print any messages.
    if len(vessel_leak_calculation.messages) > 0:
        print('Messages:')
        for message in vessel_leak_calculation.messages:
            print(message)

    if resultCode == ResultCode.SUCCESS:
        if (abs((vessel_leak_calculation.discharge_records[0].mass_flow- 18.596151455819438)/ 18.596151455819438)>1e-3):
            assert False,f'Regression failed with vessel_leak_calculation.discharge_records[0].mass_flow = {vessel_leak_calculation.discharge_records[0].mass_flow}'
        if (len(vessel_leak_calculation.discharge_records) != 2):
            assert False,f'Regression failed with len(vessel_leak_calculation.discharge_records) = {len(vessel_leak_calculation.discharge_records)}'
        if (abs((vessel_leak_calculation.discharge_records[0].final_velocity - 82.9875939245342)/82.9875939245342)>1e-3):
            assert False,f'Regression failed with vessel_leak_calculation.discharge_records[0].final_velocity = {vessel_leak_calculation.discharge_records[0].final_velocity}'
        if (abs((vessel_leak_calculation.discharge_records[0].droplet_diameter - 0.00024867091738833337)/0.00024867091738833337)>1e-3):
            assert False,f'Regression failed with vessel_leak_calculation.discharge_records[0].droplet_diameter = {vessel_leak_calculation.discharge_records[0].droplet_diameter}'
        print(f'SUCCESS: vessel_leak_calculation ({vessel_leak_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED vessel_leak_calculation with result code {resultCode}'



import os
import pathlib
import sys

# When running locally the environment variable PYPWS_RUN_LOCALLY needs to be set to True.
# Check if the environment variable is set
PYPWS_RUN_LOCALLY = os.getenv('PYPWS_RUN_LOCALLY')
if PYPWS_RUN_LOCALLY and PYPWS_RUN_LOCALLY.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')

from pypws.utilities import single_or_default

# Test to get the single or default element from an empty list.
def test_single_or_default_empty_list():
    assert single_or_default([], lambda x: x == True) is None

# Test to get the single or default element from a list where multiple elements match the predicate.
def test_single_or_default_multiple_matches():
    assert single_or_default([1, 2, 3, 3, 5], lambda x: x == 3) is None

# Test to get the single or default element from a list where the element is equal to a value.
def test_single_or_default_equal_to_a_value():
    assert single_or_default([1, 2, 3, 4, 5], lambda x: x == 3) == 3

# Test to get the single or default element from a list where the element is not equal to a value.
def test_single_or_default_not_equal_to_a_value():
    assert single_or_default([1, 2, 3, 4, 5], lambda x: x != 4) is None

# Test to get the single or default element from a list where the element is not present.    
def test_single_or_default_the_value_is_not_present():
    assert single_or_default([1, 2, 3, 4, 5], lambda x: x == 6) is None

# Test to get the single or default element from a list where the element is not present and the default value is given.
def test_single_or_default_the_value_default_given():
    assert single_or_default([1, 2, 3, 4, 5], lambda x: x == 6, 0) == 0


import os
import pathlib
import sys

# When running locally the environment variable PYPWS_RUN_LOCALLY needs to be set to True.
# Check if the environment variable is set
PYPWS_RUN_LOCALLY = os.getenv('PYPWS_RUN_LOCALLY')
if PYPWS_RUN_LOCALLY and PYPWS_RUN_LOCALLY.lower() == 'true':
    # Navigate to the PYPWS directory by searching upwards until it is found.
    current_dir = pathlib.Path(__file__).resolve()

    while current_dir.name.lower() != 'package':
        if current_dir.parent == current_dir:  # Check if the current directory is the root directory
            raise FileNotFoundError("The 'pypws' directory was not found in the path hierarchy.")
        current_dir = current_dir.parent

    # Insert the path to the pypws package into sys.path.
    sys.path.insert(0, f'{current_dir}')


from pypws.calculations import ReliefValveMinTemperatureCalculation
from pypws.entities import Material, MaterialComponent
from pypws.enums import ResultCode


def test_case_140():
    
    """
    Test for relief valve minimum temperature calculation with the following inputs

    Material = "PROPANE"
    Pressure = 1e6

    """

    # Define the material
    material = Material(
        name="PROPANE",
        components=[
            MaterialComponent(
                name="PROPANE",
                mole_fraction=1.0
            )
        ]
    )

    # Define the pressure
    pressure = 1e6

    # Create a relief valve minimum temperature calculation
    relief_valve_min_temperature_calculation = ReliefValveMinTemperatureCalculation(material = material, pressure = pressure)

    # Run the calculation
    print ('Running relief_valve_min_temperature_calculation')
    result_code = relief_valve_min_temperature_calculation.run()

    # Print any messages.
    if len(relief_valve_min_temperature_calculation.messages) > 0:
        print('Messages:')
        for message in relief_valve_min_temperature_calculation.messages:
            print(message)

    if result_code == ResultCode.SUCCESS:
        if (abs((relief_valve_min_temperature_calculation.min_temperature - 300.04391253626204)/300.04391253626204)> 1e-3):
            assert False,f'Regression failed with relief_valve_min_temperature_calculation.min_temperature = {relief_valve_min_temperature_calculation.min_temperature}'
        print(f'SUCCESS: relief_valve_min_temperature_calculation ({relief_valve_min_temperature_calculation.calculation_elapsed_time}ms)')
    else:
        assert False, f'FAILED relief_valve_min_temperature_calculation with result code {result_code}'